function [imageArray] = visionShellGaborVectorized(p, preview)

% p is a struct with the following fields
%
% p.Mx               
% p.My
% p.radius
% p.period
% p.grating_angle
% p.envelope_angle
% p.aspect_ratio
% p.adjust_axis
% p.phase
% p.sigma
% p.mean_index
% p.contrast
%
% demo
% p.radius            = 90;
% p.period            = 16;
% p.grating_angle     = -45;
% p.envelope_angle    = 0;
% p.aspect_ratio      = 1;
% p.adjust_axis       = 0;
% p.phase             = pi/2;
% p.sigma             = .25;
% p.mean_index        = 124;
% p.contrast          = 1;

        
% grab parameters
if (nargin == 1)
    preview = 0;
end

radius          = p.radius;
period          = p.period;
grating_angle   = p.grating_angle;
envelope_angle  = p.envelope_angle;
aspect_ratio    = p.aspect_ratio;
adjust_axis     = p.adjust_axis;
phase           = p.phase;
sigma           = p.sigma;
mean_index      = p.mean_index;
contrast        = p.contrast;


% get angle	
theta               = -grating_angle/180 * pi;				% angle of sinwave grating
if (adjust_axis == 0)
    envelope_theta 	= -(envelope_angle+90)/180 * pi;		% angle of gaussian envelope
else
    envelope_theta 	= -(envelope_angle)/180 * pi;			% angle of gaussian envelope
end

stdev               = radius * sigma/(sqrt(2));				% standard deviation of gaussian (in pixels)
					
imageArray=[];
Mx = radius;
My = radius;

[X,Y] = meshgrid(1:radius*2,1:radius*2);

% rotate grating
hprime = (X-Mx) * cos(theta) + (Y-My) * sin(theta);
vprime = -(X-Mx) * sin(theta) + (Y-My) * cos(theta);

% rotate gaussian
e_hprime = (X-Mx) * cos(envelope_theta) + (Y-My) * sin(envelope_theta);
e_vprime = -(X-Mx) * sin(envelope_theta) + (Y-My) * cos(envelope_theta);

%calculate the luminance
if (adjust_axis == 0)	%adjust the x axis
    imageArray = mean_index * (1 + (1 * sin(2*pi*hprime/period + pi/2+2*pi*phase) .* (exp (-.5 * ((e_hprime)/stdev).*((e_hprime)/stdev) - .5 * (((1/aspect_ratio).*(1/aspect_ratio) * (e_vprime)/stdev) .* ((e_vprime)/stdev) ))) ));
else					%adjust the y axis
    imageArray = mean_index * (1 + (1 * sin(2*pi*hprime/period + pi/2+2*pi*phase) .* (exp (-.5 * ((e_hprime)/stdev).*((e_hprime)/stdev) - .5 * (((aspect_ratio)*(aspect_ratio) * (e_vprime)/stdev) .* ((e_vprime)/stdev) ))) ));
end

imageArray = round(imageArray);

imageArray(find(imageArray < mean_index-124)) = mean_index-124;
imageArray(find(imageArray > mean_index+124)) = mean_index+124;
    
if (preview)
    imshow(imageArray/255);
end

            